from sqlalchemy import func

from . import db

class Hole(db.Model):
    __tablename__ = "gw_hole"

    hole_id = db.Column(db.Integer, primary_key=True)
    course_id = db.Column(db.Integer, db.ForeignKey('gw_golf_course.course_id'))
    zone = db.Column(db.String(length=20), nullable=False)
    alias = db.Column(db.String(length=20), nullable=False)
    deleted = db.Column(db.Boolean, default=False)
    time_created = db.Column(db.DateTime(timezone=True), server_default=func.now())
    time_updated = db.Column(db.DateTime(timezone=True), server_default=func.now())




from sqlalchemy import func
from . import db


class GolfCourse(db.Model):
    __tablename__ = "gw_golf_course"

    course_id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(length=100), nullable=False)
    email = db.Column(db.String(length=100), nullable=False)
    website = db.Column(db.String(length=250), nullable=True)
    deleted = db.Column(db.Boolean, default=False)
    logo_url = db.Column(db.String(length=250))
    time_created = db.Column(db.DateTime(timezone=True), server_default=func.now())
    time_updated = db.Column(db.DateTime(timezone=True), server_default=func.now())




import datetime

from . import default


class Config(default.Config):
    """
        development config
    """
    DEBUG = True
    JWT_ACCESS_TOKEN_EXPIRES = datetime.timedelta(days=15)
    SQLALCHEMY_DATABASE_URI = "mysql://groundworx:groundworx@mysql/groundworx"






class FieldRequired(Exception):
    def __init__(self, field, errors):
        # Call the base class constructor with the parameters it needs
        message = "The field {} is required".format(field)
        super().__init__(message)
        # Now for your custom code...
        self.errors = errors


class FieldError(Exception):
    def __init__(self, field, errors):
        # Call the base class constructor with the parameters it needs
        message = "The field {} is wrong".format(field)
        super().__init__(message)
        # Now for your custom code...
        self.errors = errors



import datetime
import inspect
from inspect import getmembers

from sqlalchemy.orm.collections import InstrumentedList
from sqlalchemy.util import NoneType

from app import db


class ModelJson:
    not_show = [""]
    extras = [""]

    def keys(self):
        return [i for i in self.__dict__.keys() if i[:1] != '_']

    def __getitem__(self, key):
        return dict(zip("abc", "one two three".split()))[key]

    def _get_foreign_keys(self):
        return list(self.__table__.foreign_keys)

    def get_column_type(self, name):
        return getattr(getattr(self.__table__.columns, name), "type")

    @property
    def columns(self):
        columns = [column.name for column in self.__table__.columns]
        columns.extend(self.extras)
        return columns

    def get_methods_not_allowed(self):
        members = getmembers(self, predicate=inspect.ismethod)
        if members:
            member_names = [member[0] for member in members]
            return member_names

    def _validates(self, data):
        from sqlalchemy_utils.types.choice import Choice
        if type(data) == Choice:
            new_obj = {"code": data.code, "value": data.value}
            data = new_obj
        elif type(data) is InstrumentedList:
            new_data = []
            for obj in data:
                new_data.append(self._validates(obj))
            data = new_data
        elif str(type(data)) == "<class 'sqlalchemy.orm.dynamic.AppenderBaseQuery'>":
            new_data = []
            for obj in data:
                new_data.append(self._validates(obj))
            data = new_data
        elif type(data) in ():
            if hasattr(data, "uuid"):
                data = data.uuid
            else:
                data = self._validates(data.to_json())
        elif type(data) in [datetime.time]:
            data = data.strftime("%H:%M:%S")
        elif type(data) is dict:
            new_data = {}
            for key, value in data.items():
                new_data[key] = self._validates(value)
            data = new_data
        elif type(data) not in (str, int, dict, list, bool, NoneType):
            data = str(data)
        return data

    def to_json(self):
        obj = {}
        a = self.id
        for key in [i for i in dir(self) if i[:1] != '_' and i not in self.not_show and
                                            i not in self.get_methods_not_allowed() and i in self.columns]:
            data = getattr(self, key)
            obj[key] = self._validates(data)

        return obj

    def patch(self, new_data):
        # Create the patch object
        columns = self.columns
        for key, value in new_data.items():
            if key in columns:
                if hasattr(self, key):
                    try:
                        setattr(self, key, value)
                    except Exception as e:
                        print(key, value, e)

        db.session.commit()



from sqlalchemy import func

from . import db


class UserType(db.Model):
    __tablename__ = "gw_user_type"

    type_id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(length=100), nullable=False)



class User(db.Model):
    __tablename__ = "gw_user"

    user_id = db.Column(db.Integer, primary_key=True)
    course_id = db.Column(db.Integer, db.ForeignKey('gw_golf_course.course_id'))
    type_id = db.Column(db.Integer, db.ForeignKey('gw_user_type.type_id'))
    name = db.Column(db.String(length=50), nullable=False)
    last = db.Column(db.String(length=50), nullable=False)
    email = db.Column(db.String(length=100), nullable=False)
    password = db.Column(db.String(length=80), nullable=False)
    phone = db.Column(db.String(length=13), nullable=False)
    status = db.Column(db.String(length=10), nullable=False)
    logo_url = db.Column(db.String(length=250))
    deleted = db.Column(db.Boolean, default=False)
    time_created = db.Column(db.DateTime(timezone=True), server_default=func.now())
    time_updated = db.Column(db.DateTime(timezone=True), server_default=func.now())


class UserToken(db.Model):
    __tablename__ = "gw_user_token"

    token_id = db.Column(db.Integer, primary_key=True)
    user_id = db.Column(db.Integer, db.ForeignKey('gw_user.user_id'))
    token = db.Column(db.String(length=40), nullable=False)
    is_active = db.Column(db.Boolean, default=True)
    time_created = db.Column(db.DateTime(timezone=True), server_default=func.now())


from sqlalchemy import func

from . import db


class Sensor(db.Model):
    __tablename__ = "gw_sensor"

    sensor_id = db.Column(db.Integer, primary_key=True)
    hole_id = db.Column(db.Integer, db.ForeignKey('gw_hole.hole_id'))
    MEID = db.Column(db.String(length=18), nullable=False)
    serial_number = db.Column(db.String(length=18), nullable=False)
    latitude = db.Column(db.DECIMAL(13, 10), nullable=True)
    longitude = db.Column(db.DECIMAL(13, 10), nullable=True)
    status = db.Column(db.String(10))
    alias = db.Column(db.String(20))
    deleted = db.Column(db.Boolean, default=False)
    time_created = db.Column(db.DateTime(timezone=True), server_default=func.now())
    time_updated = db.Column(db.DateTime(timezone=True), server_default=func.now())

